# Copyright (c) 2019 MetPy Developers.
# Distributed under the terms of the BSD 3-Clause License.
# SPDX-License-Identifier: BSD-3-Clause
"""Contains updated functions that will be modified in a future release."""

# This file left as a placeholder for function changes in the future.
