# Copyright (c) 2014,2015 MetPy Developers.
# Distributed under the terms of the BSD 3-Clause License.
# SPDX-License-Identifier: BSD-3-Clause

# flake8: noqa
# Generated file -- do not modify
descriptions = {"T1_Output_Frames": "The number of octets received on interface, including frame octets (octet)",
                "T1_Input_Frames": "The number of octets sent on interface, including frame octets (octet)",
                "Router_Memory_Used": "Bytes currently in use by applications on managed device (byte)",
                "Router_Memory_Free": "Bytes currently free on managed device (byte)",
                "Router_Memory_Utilization": "%",
                "CSU_Loss_of_Signal": "Number of times Loss of Signal event detected",
                "CSU_Loss_of_Frames": "Number of times Loss of Frames event detected",
                "CSU_Yellow_Alarms": "Number of times Resource Availability Indication (RAI) (yellow) alarm received.",
                "CSU_Blue_Alarms": "Number of times Alarm Indication Signal (AIS) (blue) alarm received.",
                "CSU_24hr_Errored_Seconds": "Number of errored seconds in previous 24 hours. (s)",
                "CSU_24hr_Severely_Errored_Seconds": "Number of severely errored seconds in previous 24 hours. (s)",
                "CSU_24hr_Severely_Errored_Framing_Seconds": "Number of severely errored framing seconds in previous 24 hours. (s)",
                "CSU_24hr_Unavailable_Seconds": "Number of unavailable seconds in previous 24 hours. (s)",
                "CSU_24hr_Controlled_Slip_Seconds": "Number of controlled slip seconds in previous 24 hours. (s)",
                "CSU_24hr_Path_Coding_Violations": "Number of path coding violations in previous 24 hours.",
                "CSU_24hr_Line_Errored_Seconds": "Number of line errored seconds in previous 24 hours. (s)",
                "CSU_24hr_Bursty_Errored_Seconds": "Number of bursty errored seconds in previous 24 hours. (s)",
                "CSU_24hr_Degraded_Minutes": "Number of degraded minutes in previous 24 hours. (min)",
                "LAN_Switch_Memory_Used": "Bytes currently in use by applications on this device (byte)",
                "LAN_Switch_Memory_Free": "Bytes currently free on this device (byte)",
                "LAN_Switch_Memory_Utilization": "%",
                "NTP_Rejected_Packets": "Number of packets rejected by NTP application layer",
                "NTP_Estimated_Time_Error": "Current estimated time error of the time server (usec)",
                "GPS_Satellites": "Current number of GPS satellites used in position and time fix calculation",
                "GPS_Max_Signal_Strength": "Strongest signal strength of all tracking satellites as seen by receiver (dB)",
                "IPC_Status": "Status of the communications between channels on a redundant system. N/A on a Single channel system.",
                "Commanded_Channel_Control": "Indicates which channel the RDA has commanded to be the controlling channel. This is not necessarily the channel which is in control.",
                "DAU_Test_0": "Tests the performance of the DAU A/D Mutiplexer with a known low voltage input.",
                "DAU_Test_1": "Tests the performance of the DAU A/D Mutiplexer with a known medium voltage input.",
                "DAU_Test_2": "Tests the performance of the DAU A/D Mutiplexer with a known high voltage input.",
                "AME_Internal_Temperature": "deg C",
                "AME_Receiver_Module_Temperature": "deg C",
                "AME_BITE_CAL_Module_Temperature": "deg C",
                "AME_Peltier_Pulse_Width_Modulation": "%",
                "AME_pos_3_3V_PS_Voltage": "V",
                "AME_pos_5V_PS_Voltage": "V",
                "AME_pos_6_5V_PS_Voltage": "V",
                "AME_pos_15V_PS_Voltage": "V",
                "AME_pos_48V_PS_Voltage": "V",
                "AME_STALO_Power": "V",
                "Peltier_Current": "A",
                "ADC_Calibration_Reference_Voltage": "V",
                "AME_Peltier_Inside_Fan_Current": "A",
                "AME_Peltier_Outside_Fan_Current": "A",
                "Horizontal_TR_Limiter_Voltage": "V",
                "Vertical_TR_Limiter_Voltage": "V",
                "ADC_Calibration_Offset_Voltage": "mV",
                "UPS_Time_on_Battery": "s",
                "UPS_Battery_Temperature": "deg C",
                "UPS_Output_Voltage": "V",
                "UPS_Output_Frequency": "Hz",
                "UPS_Output_Current": "A",
                "Power_Administrator_Load": "A",
                "Transmitter_RF_Power__Sensor": "mW",
                "Horizontal_XMTR_Peak_Power": "kW",
                "XMTR_Peak_Power": "kW",
                "Vertical_XMTR_Peak_Power": "kW",
                "XMTR_RF_Avg_Power": "W",
                "Receiver_Bias": "dB",
                "Transmit_Imbalance": "dB",
                "Equipment_Shelter_Temperature": "deg C",
                "Outside_Ambient_Temperature": "deg C",
                "Transmitter_Leaving_Air_Temp": "deg C",
                "AC_Unit__1_Discharge_Air_Temp": "deg C",
                "Generator_Shelter_Temperature": "deg C",
                "Radome_Air_Temperature": "deg C",
                "AC_Unit__2_Discharge_Air_Temp": "deg C",
                "DAU_pos_15v_PS": "V",
                "DAU_neg_15v_PS": "V",
                "DAU_pos_28v_PS": "V",
                "DAU_pos_5v_PS": "V",
                "Converted_Generator_Fuel_Level": "%",
                "pos_28v_PS": "V",
                "Pedestal_pos_15v_PS": "V",
                "Encoder_pos_5v_PS": "V",
                "Pedestal_pos_5v_PS": "V",
                "Pedestal_neg_15v_PS": "V",
                "Horizontal_Short_Pulse_Noise": "dBm",
                "Horizontal_Long_Pulse_Noise": "dBm",
                "Horizontal_Noise_Temperature": "K",
                "Vertical_Noise_Long_Pulse": "dBm",
                "Vertical_Noise_Temperature": "K",
                "Horizontal_Dynamic_Range": "dB",
                "Horizontal_Delta_dBZ0": "dB",
                "Vertical_Delta_dBZ0": "dB",
                "KD_Peak_Measured": "dBm",
                "Short_Pulse__Horizontal_dBZ0": "dBZ",
                "Long_Pulse__Horizontal_dBZ0": "dBZ",
                "Horizontal_I0": "dBm",
                "Vertical_I0": "dBm",
                "Vertical_Dynamic_Range": "dB",
                "Short_Pulse__Vertical_dBZ0": "dBZ",
                "Long_Pulse__Vertical_dBZ0": "dBZ",
                "Horizontal_Power_Sense": "dBm",
                "Vertical_Power_Sense": "dBm",
                "ZDR_Bias": "dB",
                "Clutter_Suppression_Delta": "dB",
                "Clutter_Suppression_Unfiltered_Power": "dBZ",
                "Clutter_Suppression_Filtered_Power": "dBZ",
                "Transmit_Burst_Power": "dBm",
                "Transmit_Burst_Phase": "deg"}

fields = [(None, "2x"),
          ("Loop_Back_Test_Status", "H"),
          ("T1_Output_Frames", "L"),
          ("T1_Input_Frames", "L"),
          ("Router_Memory_Used", "L"),
          ("Router_Memory_Free", "L"),
          ("Router_Memory_Utilization", "H"),
          (None, "2x"),
          ("CSU_Loss_of_Signal", "L"),
          ("CSU_Loss_of_Frames", "L"),
          ("CSU_Yellow_Alarms", "L"),
          ("CSU_Blue_Alarms", "L"),
          ("CSU_24hr_Errored_Seconds", "L"),
          ("CSU_24hr_Severely_Errored_Seconds", "L"),
          ("CSU_24hr_Severely_Errored_Framing_Seconds", "L"),
          ("CSU_24hr_Unavailable_Seconds", "L"),
          ("CSU_24hr_Controlled_Slip_Seconds", "L"),
          ("CSU_24hr_Path_Coding_Violations", "L"),
          ("CSU_24hr_Line_Errored_Seconds", "L"),
          ("CSU_24hr_Bursty_Errored_Seconds", "L"),
          ("CSU_24hr_Degraded_Minutes", "L"),
          ("LAN_Switch_Memory_Used", "L"),
          ("LAN_Switch_Memory_Free", "L"),
          ("LAN_Switch_Memory_Utilization", "H"),
          (None, "2x"),
          ("NTP_Rejected_Packets", "L"),
          ("NTP_Estimated_Time_Error", "l"),
          ("GPS_Satellites", "l"),
          ("GPS_Max_Signal_Strength", "l"),
          ("IPC_Status", "H"),
          ("Commanded_Channel_Control", "H"),
          ("DAU_Test_0", "H"),
          ("DAU_Test_1", "H"),
          ("DAU_Test_2", "H"),
          ("AME_Polarization", "H"),
          ("AME_Internal_Temperature", "f"),
          ("AME_Receiver_Module_Temperature", "f"),
          ("AME_BITE_CAL_Module_Temperature", "f"),
          ("AME_Peltier_Pulse_Width_Modulation", "H"),
          ("AME_Peltier_Status", "H"),
          ("AME_A_D_Converter_Status", "H"),
          ("AME_State", "H"),
          ("AME_pos_3_3V_PS_Voltage", "f"),
          ("AME_pos_5V_PS_Voltage", "f"),
          ("AME_pos_6_5V_PS_Voltage", "f"),
          ("AME_pos_15V_PS_Voltage", "f"),
          ("AME_pos_48V_PS_Voltage", "f"),
          ("AME_STALO_Power", "f"),
          ("Peltier_Current", "f"),
          ("ADC_Calibration_Reference_Voltage", "f"),
          ("AME_Mode", "H"),
          ("AME_Peltier_Mode", "H"),
          ("AME_Peltier_Inside_Fan_Current", "f"),
          ("AME_Peltier_Outside_Fan_Current", "f"),
          ("Horizontal_TR_Limiter_Voltage", "f"),
          ("Vertical_TR_Limiter_Voltage", "f"),
          ("ADC_Calibration_Offset_Voltage", "f"),
          ("ADC_Calibration_Gain_Correction", "f"),
          ("Power_UPS_Battery_Status", "L"),
          ("UPS_Time_on_Battery", "L"),
          ("UPS_Battery_Temperature", "f"),
          ("UPS_Output_Voltage", "f"),
          ("UPS_Output_Frequency", "f"),
          ("UPS_Output_Current", "f"),
          ("Power_Administrator_Load", "f"),
          (None, "48x"),
          ("pos_5_VDC_PS", "H"),
          ("pos_15_VDC_PS", "H"),
          ("pos_28_VDC_PS", "H"),
          ("neg_15_VDC_PS", "H"),
          ("pos_45_VDC_PS", "H"),
          ("Filament_PS_Voltage", "H"),
          ("Vacuum_Pump_PS_Voltage", "H"),
          ("Focus_Coil_PS_Voltage", "H"),
          ("Filament_PS", "H"),
          ("Klystron_Warmup", "H"),
          ("Transmitter_Available", "H"),
          ("WG_Switch_Position", "H"),
          ("WG_PFN_Transfer_Interlock", "H"),
          ("Maintenance_Mode", "H"),
          ("Maintenance_Required", "H"),
          ("PFN_Switch_Position", "H"),
          ("Modulator_Overload", "H"),
          ("Modulator_Inv_Current", "H"),
          ("Modulator_Switch_Fail", "H"),
          ("Main_Power_Voltage", "H"),
          ("Charging_System_Fail", "H"),
          ("Inverse_Diode_Current", "H"),
          ("Trigger_Amplifier", "H"),
          ("Circulator_Temperature", "H"),
          ("Spectrum_Filter_Pressure", "H"),
          ("WG_ARC_VSWR", "H"),
          ("Cabinet_Interlock", "H"),
          ("Cabinet_Air_Temperature", "H"),
          ("Cabinet_Airflow", "H"),
          ("Klystron_Current", "H"),
          ("Klystron_Filament_Current", "H"),
          ("Klystron_Vacion_Current", "H"),
          ("Klystron_Air_Temperature", "H"),
          ("Klystron_Airflow", "H"),
          ("Modulator_Switch_Maintenance", "H"),
          ("Post_Charge_Regulator_Maintenance", "H"),
          ("WG_Pressure_Humidity", "H"),
          ("Transmitter_Overvoltage", "H"),
          ("Transmitter_Overcurrent", "H"),
          ("Focus_Coil_Current", "H"),
          ("Focus_Coil_Airflow", "H"),
          ("Oil_Temperature", "H"),
          ("PRF_Limit", "H"),
          ("Transmitter_Oil_Level", "H"),
          ("Transmitter_Battery_Charging", "H"),
          ("High_Voltage__HV__Status", "H"),
          ("Transmitter_Recycling_Summary", "H"),
          ("Transmitter_Inoperable", "H"),
          ("Transmitter_Air_Filter", "H"),
          ("Zero_Test_Bit_0", "H"),
          ("Zero_Test_Bit_1", "H"),
          ("Zero_Test_Bit_2", "H"),
          ("Zero_Test_Bit_3", "H"),
          ("Zero_Test_Bit_4", "H"),
          ("Zero_Test_Bit_5", "H"),
          ("Zero_Test_Bit_6", "H"),
          ("Zero_Test_Bit_7", "H"),
          ("One_Test_Bit_0", "H"),
          ("One_Test_Bit_1", "H"),
          ("One_Test_Bit_2", "H"),
          ("One_Test_Bit_3", "H"),
          ("One_Test_Bit_4", "H"),
          ("One_Test_Bit_5", "H"),
          ("One_Test_Bit_6", "H"),
          ("One_Test_Bit_7", "H"),
          ("XMTR_DAU_Interface", "H"),
          ("Transmitter_Summary_Status", "H"),
          (None, "2x"),
          ("Transmitter_RF_Power__Sensor", "f"),
          ("Horizontal_XMTR_Peak_Power", "f"),
          ("XMTR_Peak_Power", "f"),
          ("Vertical_XMTR_Peak_Power", "f"),
          ("XMTR_RF_Avg_Power", "f"),
          ("XMTR_Power_Meter_Zero", "H"),
          (None, "2x"),
          ("XMTR_Recycle_Count", "L"),
          ("Receiver_Bias", "f"),
          ("Transmit_Imbalance", "f"),
          (None, "12x"),
          ("AC_Unit__1_Compressor_Shut_off", "H"),
          ("AC_Unit__2_Compressor_Shut_off", "H"),
          ("Generator_Maintenance_Required", "H"),
          ("Generator_Battery_Voltage", "H"),
          ("Generator_Engine", "H"),
          ("Generator_Volt_Frequency", "H"),
          ("Power_Source", "H"),
          ("Transitional_Power_Source__TPS", "H"),
          ("Generator_Auto_Run_Off_Switch", "H"),
          ("Aircraft_Hazard_Lighting", "H"),
          ("DAU_UART", "H"),
          (None, "20x"),
          ("Equipment_Shelter_Fire_Detection_System", "H"),
          ("Equipment_Shelter_Fire_Smoke", "H"),
          ("Generator_Shelter_Fire_Smoke", "H"),
          ("Utility_Voltage_Frequency", "H"),
          ("Site_Security_Alarm", "H"),
          ("Security_Equipment", "H"),
          ("Security_System", "H"),
          ("Receiver_Connected_to_Antenna", "H"),
          ("Radome_Hatch", "H"),
          ("AC_Unit__1_Filter_Dirty", "H"),
          ("AC_Unit__2_Filter_Dirty", "H"),
          ("Equipment_Shelter_Temperature", "f"),
          ("Outside_Ambient_Temperature", "f"),
          ("Transmitter_Leaving_Air_Temp", "f"),
          ("AC_Unit__1_Discharge_Air_Temp", "f"),
          ("Generator_Shelter_Temperature", "f"),
          ("Radome_Air_Temperature", "f"),
          ("AC_Unit__2_Discharge_Air_Temp", "f"),
          ("DAU_pos_15v_PS", "f"),
          ("DAU_neg_15v_PS", "f"),
          ("DAU_pos_28v_PS", "f"),
          ("DAU_pos_5v_PS", "f"),
          ("Converted_Generator_Fuel_Level", "H"),
          (None, "14x"),
          ("pos_28v_PS", "f"),
          ("Pedestal_pos_15v_PS", "f"),
          ("Encoder_pos_5v_PS", "f"),
          ("Pedestal_pos_5v_PS", "f"),
          ("Pedestal_neg_15v_PS", "f"),
          ("pos_150V_Overvoltage", "H"),
          ("pos_150V_Undervoltage", "H"),
          ("Elevation_Servo_Amp_Inhibit", "H"),
          ("Elevation_Servo_Amp_Short_Circuit", "H"),
          ("Elevation_Servo_Amp_Overtemp", "H"),
          ("Elevation_Motor_Overtemp", "H"),
          ("Elevation_Stow_Pin", "H"),
          ("Elevation_PCU_Parity", "H"),
          ("Elevation_Dead_Limit", "H"),
          ("Elevation_pos_Normal_Limit", "H"),
          ("Elevation_neg_Normal_Limit", "H"),
          ("Elevation_Encoder_Light", "H"),
          ("Elevation_Gearbox_Oil", "H"),
          ("Elevation_Handwheel", "H"),
          ("Elevation_Amp_PS", "H"),
          ("Azimuth_Servo_Amp_Inhibit", "H"),
          ("Azimuth_Servo_Amp_Short_Circuit", "H"),
          ("Azimuth_Servo_Amp_Overtemp", "H"),
          ("Azimuth_Motor_Overtemp", "H"),
          ("Azimuth_Stow_Pin", "H"),
          ("Azimuth_PCU_Parity", "H"),
          ("Azimuth_Encoder_Light", "H"),
          ("Azimuth_Gearbox_Oil", "H"),
          ("Azimuth_Bull_Gear_Oil", "H"),
          ("Azimuth_Handwheel", "H"),
          ("Azimuth_Servo_Amp_PS", "H"),
          ("Servo", "H"),
          ("Pedestal_Interlock_Switch", "H"),
          (None, "2x"),
          (None, "2x"),
          ("Self_Test_1_Status", "H"),
          ("Self_Test_2_Status", "H"),
          ("Self_Test_2_Data", "H"),
          (None, "14x"),
          ("COHO_Clock", "H"),
          ("Rf_Generator_Frequency_Select_Oscillator", "H"),
          ("Rf_Generator_RF_STALO", "H"),
          ("Rf_Generator_Phase_Shifted_COHO", "H"),
          ("pos_9v_Receiver_PS", "H"),
          ("pos_5v_Receiver_PS", "H"),
          ("O_18v_Receiver_PS", "H"),
          ("neg_9v_Receiver_PS", "H"),
          ("pos_5v_Single_Channel_RDAIU_PS", "H"),
          (None, "2x"),
          ("Horizontal_Short_Pulse_Noise", "f"),
          ("Horizontal_Long_Pulse_Noise", "f"),
          ("Horizontal_Noise_Temperature", "f"),
          (None, "4x"),
          ("Vertical_Noise_Long_Pulse", "f"),
          ("Vertical_Noise_Temperature", "f"),
          ("Horizontal_Linearity", "f"),
          ("Horizontal_Dynamic_Range", "f"),
          ("Horizontal_Delta_dBZ0", "f"),
          ("Vertical_Delta_dBZ0", "f"),
          ("KD_Peak_Measured", "f"),
          (None, "4x"),
          ("Short_Pulse__Horizontal_dBZ0", "f"),
          ("Long_Pulse__Horizontal_dBZ0", "f"),
          ("Velocity__Processed", "H"),
          ("Width__Processed", "H"),
          ("Velocity__RF_Gen", "H"),
          ("Width__RF_Gen", "H"),
          ("Horizontal_I0", "f"),
          ("Vertical_I0", "f"),
          ("Vertical_Dynamic_Range", "f"),
          ("Short_Pulse__Vertical_dBZ0", "f"),
          ("Long_Pulse__Vertical_dBZ0", "f"),
          (None, "4x"),
          (None, "4x"),
          ("Horizontal_Power_Sense", "f"),
          ("Vertical_Power_Sense", "f"),
          ("ZDR_Bias", "f"),
          (None, "12x"),
          ("Clutter_Suppression_Delta", "f"),
          ("Clutter_Suppression_Unfiltered_Power", "f"),
          ("Clutter_Suppression_Filtered_Power", "f"),
          ("Transmit_Burst_Power", "f"),
          ("Transmit_Burst_Phase", "f"),
          (None, "8x"),
          (None, "4x"),
          ("Vertical_Linearity", "f"),
          (None, "8x"),
          ("State_File_Read_Status", "H"),
          ("State_File_Write_Status", "H"),
          ("Bypass_Map_File_Read_Status", "H"),
          ("Bypass_Map_File_Write_Status", "H"),
          (None, "2x"),
          (None, "2x"),
          ("Current_Adaptation_File_Read_Status", "H"),
          ("Current_Adaptation_File_Write_Status", "H"),
          ("Censor_Zone_File_Read_Status", "H"),
          ("Censor_Zone_File_Write_Status", "H"),
          ("Remote_VCP_File_Read_Status", "H"),
          ("Remote_VCP_File_Write_Status", "H"),
          ("Baseline_Adaptation_File_Read_Status", "H"),
          (None, "2x"),
          ("Clutter_Filter_Map_File_Read_Status", "H"),
          ("Clutter_Filter_Map_File_Write_Status", "H"),
          ("General_Disk_I_O_Error", "H"),
          (None, "26x"),
          ("DAU_Comm_Status", "H"),
          ("HCI_Comm_Status", "H"),
          ("Pedestal_Comm_Status", "H"),
          ("Signal_Processor_Comm_Status", "H"),
          ("AME_Communication_Status", "H"),
          ("RMS_Link_Status", "H"),
          ("RPG_Link_Status", "H"),
          (None, "2x"),
          ("Performance_Check_Time", "L"),
          (None, "20x")]
