# Copyright (c) 2014,2015 MetPy Developers.
# Distributed under the terms of the BSD 3-Clause License.
# SPDX-License-Identifier: BSD-3-Clause

# flake8: noqa
# Generated file -- do not modify
descriptions = {"ADAP_FILE_NAME": "NAME OF ADAPTATION DATA FILE",
                "ADAP_FORMAT": "FORMAT OF ADAPTATION DATA FILE",
                "ADAP_REVISION": "REVISION NUMBER OF ADAPTATION DATA FILE",
                "ADAP_DATE": "LAST MODIFIED DATE ADAPTATION DATA FILE",
                "ADAP_TIME": "LAST MODIFIED TIME OF ADAPTATION DATA FILE",
                "K1": "AZIMUTH POSITION GAIN FACTOR (K1)",
                "AZ_LAT": "LATENCY OF DCU AZIMUTH MEASUREMENT (s)",
                "K3": "ELEVATION POSITION GAIN FACTOR (K3)",
                "EL_LAT": "LATENCY OF DCU ELEVATION MEASUREMENT (s)",
                "PARKAZ": "PEDESTAL PARK POSITION IN AZIMUTH (deg)",
                "PARKEL": "PEDESTAL PARK POSITION IN ELEVATION (deg)",
                "A_FUEL_CONV_0": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (0% HGT) (%)",
                "A_FUEL_CONV_1": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (10% HGT) (%)",
                "A_FUEL_CONV_2": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (20% HGT) (%)",
                "A_FUEL_CONV_3": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (30% HGT) (%)",
                "A_FUEL_CONV_4": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (40% HGT) (%)",
                "A_FUEL_CONV_5": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (50% HGT) (%)",
                "A_FUEL_CONV_6": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (60% HGT) (%)",
                "A_FUEL_CONV_7": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (70% HGT) (%)",
                "A_FUEL_CONV_8": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (80% HGT) (%)",
                "A_FUEL_CONV_9": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (90% HGT) (%)",
                "A_FUEL_CONV_10": "GENERATOR FUEL LEVEL HEIGHT/CAPACITY CONVERSION (100% HGT) (%)",
                "A_MIN_SHELTER_TEMP": "MINIMUM EQUIPMENT SHELTER ALARM TEMPERATURE (deg C)",
                "A_MAX_SHELTER_TEMP": "MAXIMUM EQUIPMENT SHELTER ALARM TEMPERATURE (deg C)",
                "A_MIN_SHELTER_AC_TEMP_DIFF": "MINIMUM A/C DISCHARGE AIR TEMPERATURE DIFFERENTIAL (deg C)",
                "A_MAX_XMTR_AIR_TEMP": "MAXIMUM TRANSMITTER LEAVING AIR ALARM TEMPERATURE (deg C)",
                "A_MAX_RAD_TEMP": "MAXIMUM RADOME ALARM TEMPERATURE (deg C)",
                "A_MAX_RAD_TEMP_RISE": "MAXIMUM RADOME MINUS AMBIENT TEMPERATURE DIFFERENCE (deg C)",
                "PED_28V_REG_LIM": "PEDESTAL +28 VOLT POWER SUPPLY TOLERANCE (%)",
                "PED_5V_REG_LIM": "PEDESTAL +5 VOLT POWER SUPPLY TOLERANCE (%)",
                "PED_15V_REG_LIM": "PEDESTAL +/- 15 VOLT POWER SUPPLY TOLERANCE (%)",
                "A_MIN_GEN_ROOM_TEMP": "MINIMUM GENERATOR SHELTER ALARM TEMPERATURE (deg C)",
                "A_MAX_GEN_ROOM_TEMP": "MAXIMUM GENERATOR SHELTER ALARM TEMPERATURE (deg C)",
                "DAU_5V_REG_LIM": "DAU +5 VOLT POWER SUPPLY TOLERANCE (%)",
                "DAU_15V_REG_LIM": "DAU +/- 15 VOLT POWER SUPPLY TOLERANCE (%)",
                "DAU_28V_REG_LIM": "DAU +28 VOLT POWER (%)",
                "EN_5V_REG_LIM": "ENCODER +5 VOLT POWER SUPPLY TOLERANCE (%)",
                "EN_5V_NOM_VOLTS": "ENCODER +5 VOLT POWER SUPPLY NOMINAL VOLTAGE (V)",
                "RPG_CO_LOCATED": "RPG CO-LOCATED",
                "SPEC_FILTER_INSTALLED": "TRANSMITTER SPECTRUM FILTER INSTALLED",
                "TPS_INSTALLED": "TRANSITION POWER SOURCE INSTALLED",
                "RMS_INSTALLED": "FAA RMS INSTALLED",
                "A_HVDL_TST_INT": "PERFORMANCE TEST INTERVAL (h)",
                "A_RPG_LT_INT": "RPG LOOP TEST INTERVAL (min)",
                "A_MIN_STAB_UTIL_PWR_TIME": "REQUIRED INTERVAL TIME FOR STABLE UTILITY POWER (min)",
                "A_GEN_AUTO_EXER_INTERVAL": "MAXIMUM GENERATOR AUTOMATIC EXERCISE INTERVAL (h)",
                "A_UTIL_PWR_SW_REQ_INTERVAL": "RECOMMENDED SWITCH TO UTILITY POWER TIME INTERVAL (min)",
                "A_LOW_FUEL_LEVEL": "LOW FUEL TANK WARNING LEVEL (%)",
                "CONFIG_CHAN_NUMBER": "CONFIGURATION CHANNEL NUMBER",
                "A_RPG_LINK_TYPE": "RPG WIDEBAND LINK TYPE (0 = DIRECT, 1 = MICROWAVE, 2 = FIBER OPTIC)",
                "REDUNDANT_CHAN_CONFIG": "REDUNDANT CHANNEL CONFIGURATION (1 = SINGLE CHAN, 2 = FAA, 3 = NWS REDUNDANT)",
                "ATTEN_TABLE_0": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (0dB) (dB)",
                "ATTEN_TABLE_1": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (1dB) (dB)",
                "ATTEN_TABLE_2": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (2dB) (dB)",
                "ATTEN_TABLE_3": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (3dB) (dB)",
                "ATTEN_TABLE_4": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (4dB) (dB)",
                "ATTEN_TABLE_5": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (5dB) (dB)",
                "ATTEN_TABLE_6": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (6dB) (dB)",
                "ATTEN_TABLE_7": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (7dB) (dB)",
                "ATTEN_TABLE_8": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (8dB) (dB)",
                "ATTEN_TABLE_9": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (9dB) (dB)",
                "ATTEN_TABLE_10": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (10dB) (dB)",
                "ATTEN_TABLE_11": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (11dB) (dB)",
                "ATTEN_TABLE_12": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (12dB) (dB)",
                "ATTEN_TABLE_13": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (13dB) (dB)",
                "ATTEN_TABLE_14": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (14dB) (dB)",
                "ATTEN_TABLE_15": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (15dB) (dB)",
                "ATTEN_TABLE_16": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (16dB) (dB)",
                "ATTEN_TABLE_17": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (17dB) (dB)",
                "ATTEN_TABLE_18": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (18dB) (dB)",
                "ATTEN_TABLE_19": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (19dB) (dB)",
                "ATTEN_TABLE_20": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (20dB) (dB)",
                "ATTEN_TABLE_21": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (21dB) (dB)",
                "ATTEN_TABLE_22": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (22dB) (dB)",
                "ATTEN_TABLE_23": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (23dB) (dB)",
                "ATTEN_TABLE_24": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (24dB) (dB)",
                "ATTEN_TABLE_25": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (25dB) (dB)",
                "ATTEN_TABLE_26": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (26dB) (dB)",
                "ATTEN_TABLE_27": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (27dB) (dB)",
                "ATTEN_TABLE_28": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (28dB) (dB)",
                "ATTEN_TABLE_29": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (29dB) (dB)",
                "ATTEN_TABLE_30": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (30dB) (dB)",
                "ATTEN_TABLE_31": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (31dB) (dB)",
                "ATTEN_TABLE_32": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (32dB) (dB)",
                "ATTEN_TABLE_33": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (33dB) (dB)",
                "ATTEN_TABLE_34": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (34dB) (dB)",
                "ATTEN_TABLE_35": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (35dB) (dB)",
                "ATTEN_TABLE_36": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (36dB) (dB)",
                "ATTEN_TABLE_37": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (37dB) (dB)",
                "ATTEN_TABLE_38": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (38dB) (dB)",
                "ATTEN_TABLE_39": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (39dB) (dB)",
                "ATTEN_TABLE_40": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (40dB) (dB)",
                "ATTEN_TABLE_41": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (41dB) (dB)",
                "ATTEN_TABLE_42": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (42dB) (dB)",
                "ATTEN_TABLE_43": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (43dB) (dB)",
                "ATTEN_TABLE_44": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (44dB) (dB)",
                "ATTEN_TABLE_45": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (45dB) (dB)",
                "ATTEN_TABLE_46": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (46dB) (dB)",
                "ATTEN_TABLE_47": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (47dB) (dB)",
                "ATTEN_TABLE_48": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (48dB) (dB)",
                "ATTEN_TABLE_49": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (49dB) (dB)",
                "ATTEN_TABLE_50": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (50dB) (dB)",
                "ATTEN_TABLE_51": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (51dB) (dB)",
                "ATTEN_TABLE_52": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (52dB) (dB)",
                "ATTEN_TABLE_53": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (53dB) (dB)",
                "ATTEN_TABLE_54": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (54dB) (dB)",
                "ATTEN_TABLE_55": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (55dB) (dB)",
                "ATTEN_TABLE_56": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (56dB) (dB)",
                "ATTEN_TABLE_57": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (57dB) (dB)",
                "ATTEN_TABLE_58": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (58dB) (dB)",
                "ATTEN_TABLE_59": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (59dB) (dB)",
                "ATTEN_TABLE_60": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (60dB) (dB)",
                "ATTEN_TABLE_61": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (61dB) (dB)",
                "ATTEN_TABLE_62": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (62dB) (dB)",
                "ATTEN_TABLE_63": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (63dB) (dB)",
                "ATTEN_TABLE_64": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (64dB) (dB)",
                "ATTEN_TABLE_65": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (65dB) (dB)",
                "ATTEN_TABLE_66": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (66dB) (dB)",
                "ATTEN_TABLE_67": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (67dB) (dB)",
                "ATTEN_TABLE_68": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (68dB) (dB)",
                "ATTEN_TABLE_69": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (69dB) (dB)",
                "ATTEN_TABLE_70": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (70dB) (dB)",
                "ATTEN_TABLE_71": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (71dB) (dB)",
                "ATTEN_TABLE_72": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (72dB) (dB)",
                "ATTEN_TABLE_73": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (73dB) (dB)",
                "ATTEN_TABLE_74": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (74dB) (dB)",
                "ATTEN_TABLE_75": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (75dB) (dB)",
                "ATTEN_TABLE_76": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (76dB) (dB)",
                "ATTEN_TABLE_77": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (77dB) (dB)",
                "ATTEN_TABLE_78": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (78dB) (dB)",
                "ATTEN_TABLE_79": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (79dB) (dB)",
                "ATTEN_TABLE_80": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (80dB) (dB)",
                "ATTEN_TABLE_81": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (81dB) (dB)",
                "ATTEN_TABLE_82": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (82dB) (dB)",
                "ATTEN_TABLE_83": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (83dB) (dB)",
                "ATTEN_TABLE_84": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (84dB) (dB)",
                "ATTEN_TABLE_85": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (85dB) (dB)",
                "ATTEN_TABLE_86": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (86dB) (dB)",
                "ATTEN_TABLE_87": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (87dB) (dB)",
                "ATTEN_TABLE_88": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (88dB) (dB)",
                "ATTEN_TABLE_89": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (89dB) (dB)",
                "ATTEN_TABLE_90": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (90dB) (dB)",
                "ATTEN_TABLE_91": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (91dB) (dB)",
                "ATTEN_TABLE_92": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (92dB) (dB)",
                "ATTEN_TABLE_93": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (93dB) (dB)",
                "ATTEN_TABLE_94": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (94dB) (dB)",
                "ATTEN_TABLE_95": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (95dB) (dB)",
                "ATTEN_TABLE_96": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (96dB) (dB)",
                "ATTEN_TABLE_97": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (97dB) (dB)",
                "ATTEN_TABLE_98": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (98dB) (dB)",
                "ATTEN_TABLE_99": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (99dB) (dB)",
                "ATTEN_TABLE_100": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (100dB) (dB)",
                "ATTEN_TABLE_101": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (101dB) (dB)",
                "ATTEN_TABLE_102": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (102dB) (dB)",
                "ATTEN_TABLE_103": "TEST SIGNAL ATTENUATOR INSERTION LOSSES (103dB) (dB)",
                "PATH_LOSSES_7": "PATH LOSS - VERTICAL IF HELIAX TO 4AT16 (dB)",
                "PATH_LOSSES_11": "PATH LOSS - 4AT14 TO RF LOG AMP 4A29J1 (dB)",
                "PATH_LOSSES_13": "PATH LOSS - 2A9A9 RF DELAY LINE (dB)",
                "PATH_LOSSES_14": "PATH LOSS - 4AT13 TO RF LOG AMP 4A29J1 (dB)",
                "PATH_LOSSES_19": "PATH LOSS - 4AT15 TO RF LOG AMP 4A29J1 (dB)",
                "PATH_LOSSES_28": "PATH LOSS - HORIZONTAL IF HELIAX TO 4AT17 (dB)",
                "PATH_LOSSES_30": "PATH LOSS - A5 ELEVATION ROTARY JOINT (dB)",
                "PATH_LOSSES_31": "PATH LOSS - ELEVATION ROTARY JOINT TO ANTENNA (dB)",
                "PATH_LOSSES_32": "PATH LOSS - WG02 HARMONIC FILTER (dB)",
                "PATH_LOSSES_33": "PATH LOSS - WAVEGUIDE KLYSTRON T0 SWITCH (dB)",
                "PATH_LOSSES_34": "PATH LOSS - 2A1A4 WAVEGUIDE CHANNEL AZIMUTH ROTARY JOINT (dB)",
                "PATH_LOSSES_35": "PATH LOSS - WG06 SPECTRUM FILTER (dB)",
                "PATH_LOSSES_36": "PATH LOSS - COAX TRANSMITTER RF DRIVE TO 4AT15 (dB)",
                "PATH_LOSSES_37": "PATH LOSS - WAVEGUIDE SWITCH TO AZIMUTH ROTARY JOINT (dB)",
                "PATH_LOSSES_38": "PATH LOSS - WAVEGUIDE SWITCH (dB)",
                "PATH_LOSSES_39": "PATH LOSS - WG04 CIRCULATOR (dB)",
                "PATH_LOSSES_40": "PATH LOSS - A6 ARC DETECTOR (dB)",
                "PATH_LOSSES_41": "PATH LOSS - 1DC1 TRANSMITTER COUPLER STRAIGHT THRU (dB)",
                "PATH_LOSSES_42": "PATH LOSS - 1DC1 TRANSMITTER COUPLER COUPLING (dB)",
                "PATH_LOSSES_43": "PATH LOSS - A33 PAD (dB)",
                "PATH_LOSSES_44": "PATH LOSS - COAX TRANSMITTER RF SAMPLE TO A33 PAD (dB)",
                "PATH_LOSSES_45": "PATH LOSS - A20J1_4 POWER SPLITTER (dB)",
                "PATH_LOSSES_46": "PATH LOSS - A20J1_3 POWER SPLITTER (dB)",
                "PATH_LOSSES_47": "PATH LOSS - A20J1_2 POWER SPLITTER (dB)",
                "H_COUPLER_CW_LOSS": "RF PALLET HORIZONTAL COUPLER TEST SIGNAL LOSS (dB)",
                "V_COUPLER_XMT_LOSS": "RF PALLET VERTICAL COUPLER TRANSMITTER LOSS (dB)",
                "PATH_LOSSES_50": "PATH LOSS - WAVEGUIDE AZIMUTH JOINT TO ELEVATION JOINT (dB)",
                "PATH_LOSSES_52": "PATH LOSS - 1AT4 TRANSMITTER COUPLER PAD (dB)",
                "V_COUPLER_CW_LOSS": "RF PALLET VERTICAL COUPLER TEST SIGNAL LOSS (dB)",
                "PATH_LOSSES_54": "PATH LOSS - 4AT13 PAD (dB)",
                "PATH_LOSSES_58": "PATH LOSS - 4AT17 ATTENUATOR (dB)",
                "PATH_LOSSES_59": "PATH LOSS - IFD IF ANTI-ALIAS FILTER (dB)",
                "PATH_LOSSES_60": "PATH LOSS - A20J1_5 POWER SPLITTER (dB)",
                "PATH_LOSSES_61": "PATH LOSS - AT5 50dB ATTENUATOR (dB)",
                "PATH_LOSSES_63": "PATH LOSS - A39 RF_IF BURST MIXER (dB)",
                "PATH_LOSSES_64": "PATH LOSS - AR1 BURST IF AMPLIFIER (dB)",
                "PATH_LOSSES_65": "PATH LOSS - IFD BURST ANTIALIAS FILTER (dB)",
                "PATH_LOSSES_66": "PATH LOSS - DC3 J1_3 6dB COUPLER, THROUGH (dB)",
                "PATH_LOSSES_67": "PATH LOSS - DC3 J1_2 6DB COUPLER, COUPLED (dB)",
                "PATH_LOSSES_68": "PATH LOSS - AT2+AT3 26dB COHO ATTENUATOR (dB)",
                "PATH_LOSSES_69": "PATH LOSS - 4DC3J1 TO 4AT14 (dB)",
                "CHAN_CAL_DIFF": "NONCONTROLLING CHANNEL CALIBRATION DIFFERENCE (dB)",
                "LOG_AMP_FACTOR_1": "RF DETECTOR LOG AMPLIFIER SCALE FACTOR FOR CONVERTING RECEIVER TEST DATA (V/dBm)",
                "LOG_AMP_FACTOR_2": "RF DETECTOR LOG AMPLIFIER BIAS FOR CONVERTING RECEIVER TEST DATA (V)",
                "V_TS_CW": "AME VERTICAL TEST SIGNAL POWER (dBm)",
                "RNSCALE_0": "RECEIVER NOISE NORMALIZATION (-1.0 deg to -0.5 deg)",
                "RNSCALE_1": "RECEIVER NOISE NORMALIZATION (-0.5 deg to 0.0 deg)",
                "RNSCALE_2": "RECEIVER NOISE NORMALIZATION (0.0 deg to 0.5 deg)",
                "RNSCALE_3": "RECEIVER NOISE NORMALIZATION (0.5 deg to 1.0 deg)",
                "RNSCALE_4": "RECEIVER NOISE NORMALIZATION (1.0 deg to 1.5 deg)",
                "RNSCALE_5": "RECEIVER NOISE NORMALIZATION (1.5 deg to 2.0 deg)",
                "RNSCALE_6": "RECEIVER NOISE NORMALIZATION (2.0 deg to 2.5 deg)",
                "RNSCALE_7": "RECEIVER NOISE NORMALIZATION (2.5 deg to 3.0 deg)",
                "RNSCALE_8": "RECEIVER NOISE NORMALIZATION (3.0 deg to 3.5 deg)",
                "RNSCALE_9": "RECEIVER NOISE NORMALIZATION (3.5 deg to 4.0 deg)",
                "RNSCALE_10": "RECEIVER NOISE NORMALIZATION (4.0 deg to 4.5 deg)",
                "RNSCALE_11": "RECEIVER NOISE NORMALIZATION (4.5 deg to 5.0 deg)",
                "RNSCALE_12": "RECEIVER NOISE NORMALIZATION (> 5.0 deg)",
                "ATMOS_0": "TWO WAY ATMOSPHERIC LOSS/KM (-1.0 deg to -0.5 deg) (dB/km)",
                "ATMOS_1": "TWO WAY ATMOSPHERIC LOSS/KM (-0.5 deg to 0.0 deg) (dB/km)",
                "ATMOS_2": "TWO WAY ATMOSPHERIC LOSS/KM (0.0 deg to 0.5 deg) (dB/km)",
                "ATMOS_3": "TWO WAY ATMOSPHERIC LOSS/KM (0.5 deg to 1.0 deg) (dB/km)",
                "ATMOS_4": "TWO WAY ATMOSPHERIC LOSS/KM (1.0 deg to 1.5 deg) (dB/km)",
                "ATMOS_5": "TWO WAY ATMOSPHERIC LOSS/KM (1.5 deg to 2.0 deg) (dB/km)",
                "ATMOS_6": "TWO WAY ATMOSPHERIC LOSS/KM (2.0 deg to 2.5 deg) (dB/km)",
                "ATMOS_7": "TWO WAY ATMOSPHERIC LOSS/KM (2.5 deg to 3.0 deg) (dB/km)",
                "ATMOS_8": "TWO WAY ATMOSPHERIC LOSS/KM (3.0 deg to 3.5 deg) (dB/km)",
                "ATMOS_9": "TWO WAY ATMOSPHERIC LOSS/KM (3.5 deg to 4.0 deg) (dB/km)",
                "ATMOS_10": "TWO WAY ATMOSPHERIC LOSS/KM (4.0 deg to 4.5 deg) (dB/km)",
                "ATMOS_11": "TWO WAY ATMOSPHERIC LOSS/KM (4.5 deg to 5.0 deg) (dB/km)",
                "ATMOS_12": "TWO WAY ATMOSPHERIC LOSS/KM (> 5.0 deg) (dB/km)",
                "EL_INDEX_0": "BYPASS MAP GENERATION ELEVATION ANGLE (0) (deg)",
                "EL_INDEX_1": "BYPASS MAP GENERATION ELEVATION ANGLE (1) (deg)",
                "EL_INDEX_2": "BYPASS MAP GENERATION ELEVATION ANGLE (2) (deg)",
                "EL_INDEX_3": "BYPASS MAP GENERATION ELEVATION ANGLE (3) (deg)",
                "EL_INDEX_4": "BYPASS MAP GENERATION ELEVATION ANGLE (4) (deg)",
                "EL_INDEX_5": "BYPASS MAP GENERATION ELEVATION ANGLE (5) (deg)",
                "EL_INDEX_6": "BYPASS MAP GENERATION ELEVATION ANGLE (6) (deg)",
                "EL_INDEX_7": "BYPASS MAP GENERATION ELEVATION ANGLE (7) (deg)",
                "EL_INDEX_8": "BYPASS MAP GENERATION ELEVATION ANGLE (8) (deg)",
                "EL_INDEX_9": "BYPASS MAP GENERATION ELEVATION ANGLE (9) (deg)",
                "EL_INDEX_10": "BYPASS MAP GENERATION ELEVATION ANGLE (10) (deg)",
                "EL_INDEX_11": "BYPASS MAP GENERATION ELEVATION ANGLE (11) (deg)",
                "TFREQ_MHZ": "TRANSMITTER FREQUENCY (MHz)",
                "BASE_DATA_TCN": "POINT CLUTTER SUPPRESSION THRESHOLD (TCN) (dB)",
                "REFL_DATA_TOVER": "RANGE UNFOLDING OVERLAY THRESHOLD (TOVER) (dB)",
                "TAR_H_DBZ0_LP": "HORIZONTAL TARGET SYSTEM CALIBRATION (dBZ0) FOR LONG PULSE (dBZ)",
                "TAR_V_DBZ0_LP": "VERTICAL TARGET SYSTEM CALIBRATION (dBZ0) FOR LONG PULSE (dBZ)",
                "INIT_PHI_DP": "INITIAL SYSTEM DIFFERENTIAL PHASE (deg)",
                "NORM_INIT_PHI_DP": "NORMALIZED INITIAL SYSTEM DIFFERENTIAL PHASE (deg)",
                "LX_LP": "MATCHED FILTER LOSS FOR LONG PULSE (dB)",
                "LX_SP": "MATCHED FILTER LOSS FOR SHORT PULSE (dB)",
                "METEOR_PARAM": "/K/**2 HYDROMETEOR REFRACTIVITY FACTOR",
                "BEAMWIDTH": "ANTENNA BEAMWIDTH (deg)",
                "ANTENNA_GAIN": "ANTENNA GAIN INCLUDING RADOME (dB)",
                "VEL_MAINT_LIMIT": "VELOCITY CHECK DELTA MAINTENANCE LIMIT (m/s)",
                "WTH_MAINT_LIMIT": "SPECTRUM WIDTH CHECK DELTA MAINTENANCE LIMIT (m/s)",
                "VEL_DEGRAD_LIMIT": "VELOCITY CHECK DELTA DEGRADE LIMIT (m/s)",
                "WTH_DEGRAD_LIMIT": "SPECTRUM WIDTH CHECK DELTA DEGRADE LIMIT (m/s)",
                "H_NOISETEMP_DGRAD_LIMIT": "HORIZONTAL SYSTEM NOISE TEMP DEGRADE LIMIT (K)",
                "H_NOISETEMP_MAINT_LIMIT": "HORIZONTAL SYSTEM NOISE TEMP MAINTENANCE LIMIT (K)",
                "V_NOISETEMP_DGRAD_LIMIT": "VERTICAL SYSTEM NOISE TEMP DEGRADE LIMIT (K)",
                "V_NOISETEMP_MAINT_LIMIT": "VERTICAL SYSTEM NOISE TEMP MAINTENANCE LIMIT (K)",
                "KLY_DEGRADE_LIMIT": "KLYS TRON OUTPUT TARGET CONSISTENCY DEGRADE LIMIT (dB)",
                "TS_COHO": "COHO POWER AT A1J4 (dBm)",
                "H_TS_CW": "AME HORIZONTAL TEST SIGNAL POWER (dBm)",
                "TS_RF_SP": "RF DRIVE TEST SIGNAL SHORT PULSE AT 3A5J4 (dBm)",
                "TS_RF_LP": "RF DRIVE TEST SIGNAL LONG PULSE AT 3A5J4 (dBm)",
                "TS_STALO": "STALO POWER AT A1J2 (dBm)",
                "AME_H_NOISE_ENR": "AME NOISE SOURCE HORIZONTAL EXCESS NOISE RATIO (dB)",
                "XMTR_PEAK_PWR_HIGH_LIMIT": "MAXIMUM TRANSMITTER PEAK POWER ALARM LEVEL (kW)",
                "XMTR_PEAK_PWR_LOW_LIMIT": "MINIMUM TRANSMITTER PEAK POWER ALARM LEVEL (kW)",
                "H_DBZ0_DELTA_LIMIT": "DIFFERENCE BETWEEN COMPUTED AND TARGET HORIZONTAL DBZ0 LIMIT (dB)",
                "THRESHOLD1": "BYPASS MAP GENERATOR NOISE THRESHOLD (dB)",
                "THRESHOLD2": "BYPASS MAP GENERATOR REJECTION RATIO THRESHOLD (dB)",
                "CLUT_SUPP_DGRAD_LIM": "CLUTTER SUPPRESSION DEGRADE LIMIT (dB)",
                "CLUT_SUPP_MAINT_LIM": "CLUTTER SUPPRESSION MAINTENANCE LIMIT (dB)",
                "RANGE0_VALUE": "TRUE RANGE AT START OF FIRST RANGE BIN (km)",
                "XMTR_PWR_MTR_SCALE": "SCALE FACTOR USED TO CONVERT TRANSMITTER POWER BYTE DATA TO WATTS (W (4))",
                "V_DBZ0_DELTA_LIMIT": "DIFFERENCE BETWEEN COMPUTED AND TARGET VERTICAL DBZ0 LIMIT (dB)",
                "TAR_H_DBZ0_SP": "HORIZONTAL TARGET SYSTEM CALIBRATION (dBZ0) FOR SHORT PULSE (dBZ)",
                "TAR_V_DBZ0_SP": "VERTICAL TARGET SYSTEM CALIBRATION (DBZ0) FOR SHORT PULSE (dBZ)",
                "DELTAPRF": "SITE PRF SET (A=1, B=2, C=3, D=4, E=5)",
                "TAU_SP": "PULSE WIDTH OF TRANSMITTER OUTPUT IN SHORT PULSE (nsec)",
                "TAU_LP": "PULSE WIDTH OF TRANSMITTER OUTPUT IN LONG PULSE (nsec)",
                "NC_DEAD_VALUE": "NUMBER OF 1/4 KM BINS OF CORRUPTED DATA AT END OF SWEEP",
                "TAU_RF_SP": "RF DRIVE PULSE WIDTH IN SHORT PULSE (nsec)",
                "TAU_RF_LP": "RF DRIVE PULSE WIDTH IN LONG PULSE MODE (nsec)",
                "SEG1LIM": "CLUTTER MAP BOUNDARY ELEVATION BETWEEN SEGMENTS 1 & 2 (deg)",
                "SLATSEC": "SITE LATITUDE - SECONDS (s)",
                "SLONSEC": "SITE LONGITUDE - SECONDS (s)",
                "SLATDEG": "SITE LATITUDE - DEGREES (deg)",
                "SLATMIN": "SITE LATITUDE - MINUTES (min)",
                "SLONDEG": "SITE LONGITUDE - DEGREES (deg)",
                "SLONMIN": "SITE LONGITUDE - MINUTES (min)",
                "SLATDIR": "SITE LATITUDE - DIRECTION",
                "SLONDIR": "SITE LONGITUDE - DIRECTION",
                "VCPAT11": "VOLUME COVERAGE PATTERN NUMBER 11 DEFINITION",
                "VCPAT21": "VOLUME COVERAGE PATTERN NUMBER 21 DEFINITION",
                "VCPAT31": "VOLUME COVERAGE PATTERN NUMBER 31 DEFINITION",
                "VCPAT32": "VOLUME COVERAGE PATTERN NUMBER 32 DEFINITION",
                "VCPAT300": "VOLUME COVERAGE PATTERN NUMBER 300 DEFINITION",
                "VCPAT301": "VOLUME COVERAGE PATTERN NUMBER 301 DEFINITION",
                "AZ_CORRECTION_FACTOR": "AZIMUTH BORESIGHT CORRECTION FACTOR (deg)",
                "EL_CORRECTION_FACTOR": "ELEVATION BORESIGHT CORRECTION FACTOR (deg)",
                "SITE_NAME": "SITE NAME DESIGNATION",
                "ANT_MANUAL_SETUP_IELMIN": "MINIMUM ELEVATION ANGLE (deg)",
                "ANT_MANUAL_SETUP_IELMAX": "MAXIMUM ELEVATION ANGLE (deg)",
                "ANT_MANUAL_SETUP_FAZVELMAX": "MAXIMUM AZIMUTH VELOCITY (deg/s)",
                "ANT_MANUAL_SETUP_FELVELMAX": "MAXIMUM ELEVATION VELOCITY (deg/s)",
                "ANT_MANUAL_SETUP_IGND_HGT": "SITE GROUND HEIGHT (ABOVE SEA LEVEL) (m)",
                "ANT_MANUAL_SETUP_IRAD_HGT": "SITE RADAR HEIGHT (ABOVE GROUND) (m)",
                "RVP8NV_IWAVEGUIDE_LENGTH": "WAVEGUIDE LENGTH (m)",
                "VEL_DATA_TOVER": "VELOCITY UNFOLDING OVERLAY THRESHOLD (dB)",
                "WIDTH_DATA_TOVER": "WIDTH UNFOLDING OVERLAY THRESHOLD (dB)",
                "DOPPLER_RANGE_START": "START RANGE FOR FIRST DOPPLER RADIAL (km)",
                "MAX_EL_INDEX": "THE MAXIMUM INDEX FOR THE EL_INDEX PARAMETERS",
                "SEG2LIM": "CLUTTER MAP BOUNDARY ELEVATION BETWEEN SEGMENTS 2 & 3. (deg)",
                "SEG3LIM": "CLUTTER MAP BOUNDARY ELEVATION BETWEEN SEGMENTS 3 & 4. (deg)",
                "SEG4LIM": "CLUTTER MAP BOUNDARY ELEVATION BETWEEN SEGMENTS 4 & 5. (deg)",
                "NBR_EL_SEGMENTS": "NUMBER OF ELEVATION SEGMENTS IN ORDA CLUTTER MAP.",
                "H_NOISE_LONG": "HORIZONTAL RECEIVER NOISE LONG PULSE (dBm)",
                "ANT_NOISE_TEMP": "ANTENNA NOISE TEMPERATURE (K)",
                "H_NOISE_SHORT": "HORIZONTAL RECEIVER NOISE SHORT PULSE (dBm)",
                "H_NOISE_TOLERANCE": "HORIZONTAL RECEIVER NOISE TOLERANCE (dB)",
                "MIN_H_DYN_RANGE": "MINIMUM HORIZONTAL DYNAMIC RANGE (dB)",
                "GEN_INSTALLED": "AUXILIARY GENERATOR INSTALLED (FAA ONLY)",
                "GEN_EXERCISE": "AUXILIARY GENERATOR AUTOMATIC EXERCISE ENABLED (FAA ONLY)",
                "V_NOISE_TOLERANCE": "VERTICAL RECEIVER NOISE TOLERANCE (dB)",
                "MIN_V_DYN_RANGE": "MINIMUM VERTICAL DYNAMIC RANGE (dB)",
                "ZDR_BIAS_DGRAD_LIM": "SYSTEM DIFFERENTIAL REFLECTIVITY BIAS DEGRADE LIMIT (dB)",
                "V_NOISE_LONG": "VERTICAL RECEIVER NOISE LONG PULSE (dBm)",
                "V_NOISE_SHORT": "VERTICAL RECEIVER NOISE SHORT PULSE (dBm)",
                "ZDR_DATA_TOVER": "ZDR UNFOLDING OVERLAY THRESHOLD (dB)",
                "PHI_DATA_TOVER": "PHI UNFOLDING OVERLAY THRESHOLD (dB)",
                "RHO_DATA_TOVER": "RHO UNFOLDING OVERLAY THRESHOLD (dB)",
                "STALO_POWER_DGRAD_LIMIT": "STALO POWER DEGRADE LIMIT (V)",
                "STALO_POWER_MAINT_LIMIT": "STALO POWER MAINTENANCE LIMIT (V)",
                "MIN_H_PWR_SENSE": "MINIMUM HORIZONTAL POWER SENSE (dBm)",
                "MIN_V_PWR_SENSE": "MINIMUM VERTICAL POWER SENSE (dBm)",
                "H_PWR_SENSE_OFFSET": "HORIZONTAL POWER SENSE CALIBRATION OFFSET (dB)",
                "V_PWR_SENSE_OFFSET": "VERTICAL POWER SENSE CALIBRATION OFFSET (dB)",
                "PS_GAIN_REF": "RF PALLET BIAS ERROR (dB)",
                "RF_PALLET_BROAD_LOSS": "RF PALLET BROADBAND LOSS (dB)",
                "ZDR_CHECK_THRESHOLD": "ZDR CHECK FAILURE THRESHOLD (dB)",
                "PHI_CHECK_THRESHOLD": "PHI CHECK FAILURE THRESHOLD (deg)",
                "RHO_CHECK_THRESHOLD": "RHO CHECK FAILURE THRESHOLD (ratio)",
                "AME_PS_TOLERANCE": "AME POWER SUPPLY TOLERANCE (%)",
                "AME_MAX_TEMP": "MAXIMUM AME INTERNAL ALARM TEMPERATURE (deg C)",
                "AME_MIN_TEMP": "MINIMUM AME INTERNAL ALARM TEMPERATURE (deg C)",
                "RCVR_MOD_MAX_TEMP": "MAXIMUM AME RECEIVER MODULE ALARM TEMPERATURE (deg C)",
                "RCVR_MOD_MIN_TEMP": "MINIMUM AME RECEIVER MODULE ALARM TEMPERATURE (deg C)",
                "BITE_MOD_MAX_TEMP": "MAXIMUM AME BITE MODULE ALARM TEMPERATURE (deg C)",
                "BITE_MOD_MIN_TEMP": "MINIMUM AME BITE MODULE ALARM TEMPERATURE (deg C)",
                "DEFAULT_POLARIZATION": "DEFAULT (H+V) MICROWAVE ASSEMBLY PHASE SHIFTER POSITION",
                "TR_LIMIT_DGRAD_LIMIT": "TR LIMITER DEGRADE LIMIT (V)",
                "TR_LIMIT_FAIL_LIMIT": "TR LIMITER FAILURE LIMIT (V)",
                "AME_CURRENT_TOLERANCE": "AME PELTIER CURRENT TOLERANCE (%)",
                "H_ONLY_POLARIZATION": "HORIZONTAL (H ONLY) MICROWAVE ASSEMBLY PHASE SHIFTER POSITION",
                "V_ONLY_POLARIZATION": "VERTICAL (V ONLY) MICROWAVE ASSEMBLY PHASE SHIFTER POSITION",
                "REFLECTOR_BIAS": "ANTENNA REFLECTOR BIAS (dB)",
                "A_MIN_SHELTER_TEMP_WARN": "LOW EQUIPMENT SHELTER TEMPERATURE WARNING LIMIT (deg C)"}

fields = [("ADAP_FILE_NAME", "12s"),
          ("ADAP_FORMAT", "4s"),
          ("ADAP_REVISION", "4s"),
          ("ADAP_DATE", "12s"),
          ("ADAP_TIME", "12s"),
          ("K1", "f"),
          ("AZ_LAT", "f"),
          ("K3", "f"),
          ("EL_LAT", "f"),
          ("PARKAZ", "f"),
          ("PARKEL", "f"),
          ("A_FUEL_CONV_0", "f"),
          ("A_FUEL_CONV_1", "f"),
          ("A_FUEL_CONV_2", "f"),
          ("A_FUEL_CONV_3", "f"),
          ("A_FUEL_CONV_4", "f"),
          ("A_FUEL_CONV_5", "f"),
          ("A_FUEL_CONV_6", "f"),
          ("A_FUEL_CONV_7", "f"),
          ("A_FUEL_CONV_8", "f"),
          ("A_FUEL_CONV_9", "f"),
          ("A_FUEL_CONV_10", "f"),
          ("A_MIN_SHELTER_TEMP", "f"),
          ("A_MAX_SHELTER_TEMP", "f"),
          ("A_MIN_SHELTER_AC_TEMP_DIFF", "f"),
          ("A_MAX_XMTR_AIR_TEMP", "f"),
          ("A_MAX_RAD_TEMP", "f"),
          ("A_MAX_RAD_TEMP_RISE", "f"),
          ("PED_28V_REG_LIM", "f"),
          ("PED_5V_REG_LIM", "f"),
          ("PED_15V_REG_LIM", "f"),
          ("A_MIN_GEN_ROOM_TEMP", "f"),
          ("A_MAX_GEN_ROOM_TEMP", "f"),
          ("DAU_5V_REG_LIM", "f"),
          ("DAU_15V_REG_LIM", "f"),
          ("DAU_28V_REG_LIM", "f"),
          ("EN_5V_REG_LIM", "f"),
          ("EN_5V_NOM_VOLTS", "f"),
          ("RPG_CO_LOCATED", "4s"),
          ("SPEC_FILTER_INSTALLED", "4s"),
          ("TPS_INSTALLED", "4s"),
          ("RMS_INSTALLED", "4s"),
          ("A_HVDL_TST_INT", "L"),
          ("A_RPG_LT_INT", "L"),
          ("A_MIN_STAB_UTIL_PWR_TIME", "L"),
          ("A_GEN_AUTO_EXER_INTERVAL", "L"),
          ("A_UTIL_PWR_SW_REQ_INTERVAL", "L"),
          ("A_LOW_FUEL_LEVEL", "f"),
          ("CONFIG_CHAN_NUMBER", "L"),
          ("A_RPG_LINK_TYPE", "L"),
          ("REDUNDANT_CHAN_CONFIG", "L"),
          ("ATTEN_TABLE_0", "f"),
          ("ATTEN_TABLE_1", "f"),
          ("ATTEN_TABLE_2", "f"),
          ("ATTEN_TABLE_3", "f"),
          ("ATTEN_TABLE_4", "f"),
          ("ATTEN_TABLE_5", "f"),
          ("ATTEN_TABLE_6", "f"),
          ("ATTEN_TABLE_7", "f"),
          ("ATTEN_TABLE_8", "f"),
          ("ATTEN_TABLE_9", "f"),
          ("ATTEN_TABLE_10", "f"),
          ("ATTEN_TABLE_11", "f"),
          ("ATTEN_TABLE_12", "f"),
          ("ATTEN_TABLE_13", "f"),
          ("ATTEN_TABLE_14", "f"),
          ("ATTEN_TABLE_15", "f"),
          ("ATTEN_TABLE_16", "f"),
          ("ATTEN_TABLE_17", "f"),
          ("ATTEN_TABLE_18", "f"),
          ("ATTEN_TABLE_19", "f"),
          ("ATTEN_TABLE_20", "f"),
          ("ATTEN_TABLE_21", "f"),
          ("ATTEN_TABLE_22", "f"),
          ("ATTEN_TABLE_23", "f"),
          ("ATTEN_TABLE_24", "f"),
          ("ATTEN_TABLE_25", "f"),
          ("ATTEN_TABLE_26", "f"),
          ("ATTEN_TABLE_27", "f"),
          ("ATTEN_TABLE_28", "f"),
          ("ATTEN_TABLE_29", "f"),
          ("ATTEN_TABLE_30", "f"),
          ("ATTEN_TABLE_31", "f"),
          ("ATTEN_TABLE_32", "f"),
          ("ATTEN_TABLE_33", "f"),
          ("ATTEN_TABLE_34", "f"),
          ("ATTEN_TABLE_35", "f"),
          ("ATTEN_TABLE_36", "f"),
          ("ATTEN_TABLE_37", "f"),
          ("ATTEN_TABLE_38", "f"),
          ("ATTEN_TABLE_39", "f"),
          ("ATTEN_TABLE_40", "f"),
          ("ATTEN_TABLE_41", "f"),
          ("ATTEN_TABLE_42", "f"),
          ("ATTEN_TABLE_43", "f"),
          ("ATTEN_TABLE_44", "f"),
          ("ATTEN_TABLE_45", "f"),
          ("ATTEN_TABLE_46", "f"),
          ("ATTEN_TABLE_47", "f"),
          ("ATTEN_TABLE_48", "f"),
          ("ATTEN_TABLE_49", "f"),
          ("ATTEN_TABLE_50", "f"),
          ("ATTEN_TABLE_51", "f"),
          ("ATTEN_TABLE_52", "f"),
          ("ATTEN_TABLE_53", "f"),
          ("ATTEN_TABLE_54", "f"),
          ("ATTEN_TABLE_55", "f"),
          ("ATTEN_TABLE_56", "f"),
          ("ATTEN_TABLE_57", "f"),
          ("ATTEN_TABLE_58", "f"),
          ("ATTEN_TABLE_59", "f"),
          ("ATTEN_TABLE_60", "f"),
          ("ATTEN_TABLE_61", "f"),
          ("ATTEN_TABLE_62", "f"),
          ("ATTEN_TABLE_63", "f"),
          ("ATTEN_TABLE_64", "f"),
          ("ATTEN_TABLE_65", "f"),
          ("ATTEN_TABLE_66", "f"),
          ("ATTEN_TABLE_67", "f"),
          ("ATTEN_TABLE_68", "f"),
          ("ATTEN_TABLE_69", "f"),
          ("ATTEN_TABLE_70", "f"),
          ("ATTEN_TABLE_71", "f"),
          ("ATTEN_TABLE_72", "f"),
          ("ATTEN_TABLE_73", "f"),
          ("ATTEN_TABLE_74", "f"),
          ("ATTEN_TABLE_75", "f"),
          ("ATTEN_TABLE_76", "f"),
          ("ATTEN_TABLE_77", "f"),
          ("ATTEN_TABLE_78", "f"),
          ("ATTEN_TABLE_79", "f"),
          ("ATTEN_TABLE_80", "f"),
          ("ATTEN_TABLE_81", "f"),
          ("ATTEN_TABLE_82", "f"),
          ("ATTEN_TABLE_83", "f"),
          ("ATTEN_TABLE_84", "f"),
          ("ATTEN_TABLE_85", "f"),
          ("ATTEN_TABLE_86", "f"),
          ("ATTEN_TABLE_87", "f"),
          ("ATTEN_TABLE_88", "f"),
          ("ATTEN_TABLE_89", "f"),
          ("ATTEN_TABLE_90", "f"),
          ("ATTEN_TABLE_91", "f"),
          ("ATTEN_TABLE_92", "f"),
          ("ATTEN_TABLE_93", "f"),
          ("ATTEN_TABLE_94", "f"),
          ("ATTEN_TABLE_95", "f"),
          ("ATTEN_TABLE_96", "f"),
          ("ATTEN_TABLE_97", "f"),
          ("ATTEN_TABLE_98", "f"),
          ("ATTEN_TABLE_99", "f"),
          ("ATTEN_TABLE_100", "f"),
          ("ATTEN_TABLE_101", "f"),
          ("ATTEN_TABLE_102", "f"),
          ("ATTEN_TABLE_103", "f"),
          (None, "24x"),
          ("PATH_LOSSES_7", "f"),
          (None, "12x"),
          ("PATH_LOSSES_11", "f"),
          (None, "4x"),
          ("PATH_LOSSES_13", "f"),
          ("PATH_LOSSES_14", "f"),
          (None, "16x"),
          ("PATH_LOSSES_19", "f"),
          (None, "32x"),
          ("PATH_LOSSES_28", "f"),
          (None, "4x"),
          ("PATH_LOSSES_30", "f"),
          ("PATH_LOSSES_31", "f"),
          ("PATH_LOSSES_32", "f"),
          ("PATH_LOSSES_33", "f"),
          ("PATH_LOSSES_34", "f"),
          ("PATH_LOSSES_35", "f"),
          ("PATH_LOSSES_36", "f"),
          ("PATH_LOSSES_37", "f"),
          ("PATH_LOSSES_38", "f"),
          ("PATH_LOSSES_39", "f"),
          ("PATH_LOSSES_40", "f"),
          ("PATH_LOSSES_41", "f"),
          ("PATH_LOSSES_42", "f"),
          ("PATH_LOSSES_43", "f"),
          ("PATH_LOSSES_44", "f"),
          ("PATH_LOSSES_45", "f"),
          ("PATH_LOSSES_46", "f"),
          ("PATH_LOSSES_47", "f"),
          ("H_COUPLER_CW_LOSS", "f"),
          ("V_COUPLER_XMT_LOSS", "f"),
          ("PATH_LOSSES_50", "f"),
          (None, "4x"),
          ("PATH_LOSSES_52", "f"),
          ("V_COUPLER_CW_LOSS", "f"),
          ("PATH_LOSSES_54", "f"),
          (None, "4x"),
          (None, "4x"),
          (None, "4x"),
          ("PATH_LOSSES_58", "f"),
          ("PATH_LOSSES_59", "f"),
          ("PATH_LOSSES_60", "f"),
          ("PATH_LOSSES_61", "f"),
          (None, "4x"),
          ("PATH_LOSSES_63", "f"),
          ("PATH_LOSSES_64", "f"),
          ("PATH_LOSSES_65", "f"),
          ("PATH_LOSSES_66", "f"),
          ("PATH_LOSSES_67", "f"),
          ("PATH_LOSSES_68", "f"),
          ("PATH_LOSSES_69", "f"),
          ("CHAN_CAL_DIFF", "f"),
          (None, "4x"),
          ("LOG_AMP_FACTOR_1", "f"),
          ("LOG_AMP_FACTOR_2", "f"),
          ("V_TS_CW", "f"),
          ("RNSCALE_0", "f"),
          ("RNSCALE_1", "f"),
          ("RNSCALE_2", "f"),
          ("RNSCALE_3", "f"),
          ("RNSCALE_4", "f"),
          ("RNSCALE_5", "f"),
          ("RNSCALE_6", "f"),
          ("RNSCALE_7", "f"),
          ("RNSCALE_8", "f"),
          ("RNSCALE_9", "f"),
          ("RNSCALE_10", "f"),
          ("RNSCALE_11", "f"),
          ("RNSCALE_12", "f"),
          ("ATMOS_0", "f"),
          ("ATMOS_1", "f"),
          ("ATMOS_2", "f"),
          ("ATMOS_3", "f"),
          ("ATMOS_4", "f"),
          ("ATMOS_5", "f"),
          ("ATMOS_6", "f"),
          ("ATMOS_7", "f"),
          ("ATMOS_8", "f"),
          ("ATMOS_9", "f"),
          ("ATMOS_10", "f"),
          ("ATMOS_11", "f"),
          ("ATMOS_12", "f"),
          ("EL_INDEX_0", "f"),
          ("EL_INDEX_1", "f"),
          ("EL_INDEX_2", "f"),
          ("EL_INDEX_3", "f"),
          ("EL_INDEX_4", "f"),
          ("EL_INDEX_5", "f"),
          ("EL_INDEX_6", "f"),
          ("EL_INDEX_7", "f"),
          ("EL_INDEX_8", "f"),
          ("EL_INDEX_9", "f"),
          ("EL_INDEX_10", "f"),
          ("EL_INDEX_11", "f"),
          ("TFREQ_MHZ", "L"),
          ("BASE_DATA_TCN", "f"),
          ("REFL_DATA_TOVER", "f"),
          ("TAR_H_DBZ0_LP", "f"),
          ("TAR_V_DBZ0_LP", "f"),
          ("INIT_PHI_DP", "L"),
          ("NORM_INIT_PHI_DP", "L"),
          ("LX_LP", "f"),
          ("LX_SP", "f"),
          ("METEOR_PARAM", "f"),
          ("BEAMWIDTH", "f"),
          ("ANTENNA_GAIN", "f"),
          (None, "4x"),
          ("VEL_MAINT_LIMIT", "f"),
          ("WTH_MAINT_LIMIT", "f"),
          ("VEL_DEGRAD_LIMIT", "f"),
          ("WTH_DEGRAD_LIMIT", "f"),
          ("H_NOISETEMP_DGRAD_LIMIT", "f"),
          ("H_NOISETEMP_MAINT_LIMIT", "f"),
          ("V_NOISETEMP_DGRAD_LIMIT", "f"),
          ("V_NOISETEMP_MAINT_LIMIT", "f"),
          ("KLY_DEGRADE_LIMIT", "f"),
          ("TS_COHO", "f"),
          ("H_TS_CW", "f"),
          ("TS_RF_SP", "f"),
          ("TS_RF_LP", "f"),
          ("TS_STALO", "f"),
          ("AME_H_NOISE_ENR", "f"),
          ("XMTR_PEAK_PWR_HIGH_LIMIT", "f"),
          ("XMTR_PEAK_PWR_LOW_LIMIT", "f"),
          ("H_DBZ0_DELTA_LIMIT", "f"),
          ("THRESHOLD1", "f"),
          ("THRESHOLD2", "f"),
          ("CLUT_SUPP_DGRAD_LIM", "f"),
          ("CLUT_SUPP_MAINT_LIM", "f"),
          ("RANGE0_VALUE", "f"),
          ("XMTR_PWR_MTR_SCALE", "f"),
          ("V_DBZ0_DELTA_LIMIT", "f"),
          ("TAR_H_DBZ0_SP", "f"),
          ("TAR_V_DBZ0_SP", "f"),
          ("DELTAPRF", "L"),
          (None, "4x"),
          (None, "4x"),
          ("TAU_SP", "L"),
          ("TAU_LP", "L"),
          ("NC_DEAD_VALUE", "L"),
          ("TAU_RF_SP", "L"),
          ("TAU_RF_LP", "L"),
          ("SEG1LIM", "f"),
          ("SLATSEC", "f"),
          ("SLONSEC", "f"),
          (None, "4x"),
          ("SLATDEG", "L"),
          ("SLATMIN", "L"),
          ("SLONDEG", "L"),
          ("SLONMIN", "L"),
          ("SLATDIR", "4s"),
          ("SLONDIR", "4s"),
          (None, "4x"),
          ("VCPAT11", "1172s"),
          ("VCPAT21", "1172s"),
          ("VCPAT31", "1172s"),
          ("VCPAT32", "1172s"),
          ("VCPAT300", "1172s"),
          ("VCPAT301", "1172s"),
          ("AZ_CORRECTION_FACTOR", "f"),
          ("EL_CORRECTION_FACTOR", "f"),
          ("SITE_NAME", "4s"),
          ("ANT_MANUAL_SETUP_IELMIN", "l"),
          ("ANT_MANUAL_SETUP_IELMAX", "L"),
          ("ANT_MANUAL_SETUP_FAZVELMAX", "L"),
          ("ANT_MANUAL_SETUP_FELVELMAX", "L"),
          ("ANT_MANUAL_SETUP_IGND_HGT", "L"),
          ("ANT_MANUAL_SETUP_IRAD_HGT", "L"),
          (None, "300x"),
          ("RVP8NV_IWAVEGUIDE_LENGTH", "L"),
          (None, "44x"),
          ("VEL_DATA_TOVER", "f"),
          ("WIDTH_DATA_TOVER", "f"),
          (None, "12x"),
          ("DOPPLER_RANGE_START", "f"),
          ("MAX_EL_INDEX", "L"),
          ("SEG2LIM", "f"),
          ("SEG3LIM", "f"),
          ("SEG4LIM", "f"),
          ("NBR_EL_SEGMENTS", "L"),
          ("H_NOISE_LONG", "f"),
          ("ANT_NOISE_TEMP", "f"),
          ("H_NOISE_SHORT", "f"),
          ("H_NOISE_TOLERANCE", "f"),
          ("MIN_H_DYN_RANGE", "f"),
          ("GEN_INSTALLED", "4s"),
          ("GEN_EXERCISE", "4s"),
          ("V_NOISE_TOLERANCE", "f"),
          ("MIN_V_DYN_RANGE", "f"),
          ("ZDR_BIAS_DGRAD_LIM", "f"),
          (None, "4x"),
          (None, "4x"),
          (None, "8x"),
          ("V_NOISE_LONG", "f"),
          ("V_NOISE_SHORT", "f"),
          ("ZDR_DATA_TOVER", "f"),
          ("PHI_DATA_TOVER", "f"),
          ("RHO_DATA_TOVER", "f"),
          ("STALO_POWER_DGRAD_LIMIT", "f"),
          ("STALO_POWER_MAINT_LIMIT", "f"),
          ("MIN_H_PWR_SENSE", "f"),
          ("MIN_V_PWR_SENSE", "f"),
          ("H_PWR_SENSE_OFFSET", "f"),
          ("V_PWR_SENSE_OFFSET", "f"),
          ("PS_GAIN_REF", "f"),
          ("RF_PALLET_BROAD_LOSS", "f"),
          ("ZDR_CHECK_THRESHOLD", "f"),
          ("PHI_CHECK_THRESHOLD", "f"),
          ("RHO_CHECK_THRESHOLD", "f"),
          (None, "52x"),
          ("AME_PS_TOLERANCE", "f"),
          ("AME_MAX_TEMP", "f"),
          ("AME_MIN_TEMP", "f"),
          ("RCVR_MOD_MAX_TEMP", "f"),
          ("RCVR_MOD_MIN_TEMP", "f"),
          ("BITE_MOD_MAX_TEMP", "f"),
          ("BITE_MOD_MIN_TEMP", "f"),
          ("DEFAULT_POLARIZATION", "L"),
          ("TR_LIMIT_DGRAD_LIMIT", "f"),
          ("TR_LIMIT_FAIL_LIMIT", "f"),
          (None, "4x"),
          (None, "4x"),
          ("AME_CURRENT_TOLERANCE", "f"),
          ("H_ONLY_POLARIZATION", "L"),
          ("V_ONLY_POLARIZATION", "L"),
          (None, "8x"),
          ("REFLECTOR_BIAS", "f"),
          ("A_MIN_SHELTER_TEMP_WARN", "f"),
          (None, "432x")]
